# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class HmacAlgorithmEnum(str, Enum):
    """
    * `sha1` - SHA1 * `sha256` - SHA256 * `sha512` - SHA512
    """

    """
    allowed enum values
    """
    SHA1 = 'sha1'
    SHA256 = 'sha256'
    SHA512 = 'sha512'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of HmacAlgorithmEnum from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.action_run_request import ActionRunRequest

class TestActionRunRequest(unittest.TestCase):
    """ActionRunRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ActionRunRequest:
        """Test ActionRunRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ActionRunRequest`
        """
        model = ActionRunRequest()
        if include_optional:
            return ActionRunRequest(
                selected_rows = [
                    ''
                    ],
                selected_all = True,
                filter_params = {
                    'key' : null
                    },
                user_input = {
                    'key' : null
                    },
                connection_id = '',
                language = 'python'
            )
        else:
            return ActionRunRequest(
        )
        """

    def testActionRunRequest(self):
        """Test ActionRunRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr
from typing import Any, Optional
from typing_extensions import Annotated
from automio.models.integration import Integration
from automio.models.invoke_parallel_call200_response import InvokeParallelCall200Response
from automio.models.invoke_parallel_call_for_connection200_response import InvokeParallelCallForConnection200Response
from automio.models.invoke_parallel_call_for_connection_request import InvokeParallelCallForConnectionRequest
from automio.models.invoke_parallel_call_request import InvokeParallelCallRequest

from automio.api_client import ApiClient, RequestSerialized
from automio.api_response import ApiResponse
from automio.rest import RESTResponseType


class IntegrationsAPICallsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def integrations_call_create(
        self,
        integration_id: StrictStr,
        integration: Integration,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Integration:
        """Make a basic API call to the integration's endpoint

        Make a basic API call to the integration's endpoint using the specified method and parameters.

        :param integration_id: (required)
        :type integration_id: str
        :param integration: (required)
        :type integration: Integration
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._integrations_call_create_serialize(
            integration_id=integration_id,
            integration=integration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Integration",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def integrations_call_create_with_http_info(
        self,
        integration_id: StrictStr,
        integration: Integration,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Integration]:
        """Make a basic API call to the integration's endpoint

        Make a basic API call to the integration's endpoint using the specified method and parameters.

        :param integration_id: (required)
        :type integration_id: str
        :param integration: (required)
        :type integration: Integration
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._integrations_call_create_serialize(
            integration_id=integration_id,
            integration=integration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Integration",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def integrations_call_create_without_preload_content(
        self,
        integration_id: StrictStr,
        integration: Integration,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Make a basic API call to the integration's endpoint

        Make a basic API call to the integration's endpoint using the specified method and parameters.

        :param integration_id: (required)
        :type integration_id: str
        :param integration: (required)
        :type integration: Integration
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._integrations_call_create_serialize(
            integration_id=integration_id,
            integration=integration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Integration",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _integrations_call_create_serialize(
        self,
        integration_id,
        integration,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration_id is not None:
            _path_params['integration_id'] = integration_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if integration is not None:
            _body_params = integration


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json', 
                        'application/x-www-form-urlencoded', 
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'knoxApiToken'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/integrations/{integration_id}/call/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def invoke_parallel_call(
        self,
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_request: Optional[InvokeParallelCallRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> InvokeParallelCall200Response:
        """Invoke parallel API calls with connection IDs per data object

        Make parallel API calls where each data object contains its own connection_id. Requests are grouped by connection and executed in parallel within each group.

        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_request:
        :type invoke_parallel_call_request: InvokeParallelCallRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_serialize(
            integration_id=integration_id,
            x_app_id=x_app_id,
            table=table,
            invoke_parallel_call_request=invoke_parallel_call_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCall200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def invoke_parallel_call_with_http_info(
        self,
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_request: Optional[InvokeParallelCallRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[InvokeParallelCall200Response]:
        """Invoke parallel API calls with connection IDs per data object

        Make parallel API calls where each data object contains its own connection_id. Requests are grouped by connection and executed in parallel within each group.

        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_request:
        :type invoke_parallel_call_request: InvokeParallelCallRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_serialize(
            integration_id=integration_id,
            x_app_id=x_app_id,
            table=table,
            invoke_parallel_call_request=invoke_parallel_call_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCall200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def invoke_parallel_call_without_preload_content(
        self,
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_request: Optional[InvokeParallelCallRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Invoke parallel API calls with connection IDs per data object

        Make parallel API calls where each data object contains its own connection_id. Requests are grouped by connection and executed in parallel within each group.

        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_request:
        :type invoke_parallel_call_request: InvokeParallelCallRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_serialize(
            integration_id=integration_id,
            x_app_id=x_app_id,
            table=table,
            invoke_parallel_call_request=invoke_parallel_call_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCall200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _invoke_parallel_call_serialize(
        self,
        integration_id,
        x_app_id,
        table,
        invoke_parallel_call_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration_id is not None:
            _path_params['integration_id'] = integration_id
        # process the query parameters
        if table is not None:
            
            _query_params.append(('table', table))
            
        # process the header parameters
        if x_app_id is not None:
            _header_params['x-app-id'] = x_app_id
        # process the form parameters
        # process the body parameter
        if invoke_parallel_call_request is not None:
            _body_params = invoke_parallel_call_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'knoxApiToken'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/integrations/{integration_id}/parallel_call/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def invoke_parallel_call_for_connection(
        self,
        connection_id: Annotated[Any, Field(description="Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username).")],
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        organization_id: Annotated[Optional[Any], Field(description="External organization ID to use for the API calls. If not provided, the user's organization will be used.")] = None,
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_for_connection_request: Optional[InvokeParallelCallForConnectionRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> InvokeParallelCallForConnection200Response:
        """Invoke a parallel API call for each data object in POST.data_list

        Forward the request to the integration's parallel call endpoint to initiate request for every data object in POST.data_list.

        :param connection_id: Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username). (required)
        :type connection_id: InvokeParallelCallTableParameter
        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param organization_id: External organization ID to use for the API calls. If not provided, the user's organization will be used.
        :type organization_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_for_connection_request:
        :type invoke_parallel_call_for_connection_request: InvokeParallelCallForConnectionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_for_connection_serialize(
            connection_id=connection_id,
            integration_id=integration_id,
            x_app_id=x_app_id,
            organization_id=organization_id,
            table=table,
            invoke_parallel_call_for_connection_request=invoke_parallel_call_for_connection_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCallForConnection200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def invoke_parallel_call_for_connection_with_http_info(
        self,
        connection_id: Annotated[Any, Field(description="Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username).")],
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        organization_id: Annotated[Optional[Any], Field(description="External organization ID to use for the API calls. If not provided, the user's organization will be used.")] = None,
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_for_connection_request: Optional[InvokeParallelCallForConnectionRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[InvokeParallelCallForConnection200Response]:
        """Invoke a parallel API call for each data object in POST.data_list

        Forward the request to the integration's parallel call endpoint to initiate request for every data object in POST.data_list.

        :param connection_id: Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username). (required)
        :type connection_id: InvokeParallelCallTableParameter
        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param organization_id: External organization ID to use for the API calls. If not provided, the user's organization will be used.
        :type organization_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_for_connection_request:
        :type invoke_parallel_call_for_connection_request: InvokeParallelCallForConnectionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_for_connection_serialize(
            connection_id=connection_id,
            integration_id=integration_id,
            x_app_id=x_app_id,
            organization_id=organization_id,
            table=table,
            invoke_parallel_call_for_connection_request=invoke_parallel_call_for_connection_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCallForConnection200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def invoke_parallel_call_for_connection_without_preload_content(
        self,
        connection_id: Annotated[Any, Field(description="Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username).")],
        integration_id: StrictStr,
        x_app_id: Annotated[Any, Field(description="The ID of the application")],
        organization_id: Annotated[Optional[Any], Field(description="External organization ID to use for the API calls. If not provided, the user's organization will be used.")] = None,
        table: Annotated[Optional[Any], Field(description="Table name to use for account resolution and validation.")] = None,
        invoke_parallel_call_for_connection_request: Optional[InvokeParallelCallForConnectionRequest] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Invoke a parallel API call for each data object in POST.data_list

        Forward the request to the integration's parallel call endpoint to initiate request for every data object in POST.data_list.

        :param connection_id: Unique identifier for the integration provider account. Defaults to the primary key of the account object. Alternative identifiers may be used based on authentication type: OAuth (email or UID), API Keys (key name), or Basic Authentication (username). (required)
        :type connection_id: InvokeParallelCallTableParameter
        :param integration_id: (required)
        :type integration_id: str
        :param x_app_id: The ID of the application (required)
        :type x_app_id: InvokeParallelCallTableParameter
        :param organization_id: External organization ID to use for the API calls. If not provided, the user's organization will be used.
        :type organization_id: InvokeParallelCallTableParameter
        :param table: Table name to use for account resolution and validation.
        :type table: InvokeParallelCallTableParameter
        :param invoke_parallel_call_for_connection_request:
        :type invoke_parallel_call_for_connection_request: InvokeParallelCallForConnectionRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._invoke_parallel_call_for_connection_serialize(
            connection_id=connection_id,
            integration_id=integration_id,
            x_app_id=x_app_id,
            organization_id=organization_id,
            table=table,
            invoke_parallel_call_for_connection_request=invoke_parallel_call_for_connection_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "InvokeParallelCallForConnection200Response",
            '400': "object",
            '404': "object",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _invoke_parallel_call_for_connection_serialize(
        self,
        connection_id,
        integration_id,
        x_app_id,
        organization_id,
        table,
        invoke_parallel_call_for_connection_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if integration_id is not None:
            _path_params['integration_id'] = integration_id
        # process the query parameters
        if connection_id is not None:
            
            _query_params.append(('connection_id', connection_id))
            
        if organization_id is not None:
            
            _query_params.append(('organization_id', organization_id))
            
        if table is not None:
            
            _query_params.append(('table', table))
            
        # process the header parameters
        if x_app_id is not None:
            _header_params['x-app-id'] = x_app_id
        # process the form parameters
        # process the body parameter
        if invoke_parallel_call_for_connection_request is not None:
            _body_params = invoke_parallel_call_for_connection_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'knoxApiToken'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/integrations/{integration_id}/parallel_call_for_connection/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

# flake8: noqa
"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from automio.models.action import Action
from automio.models.action_run_request import ActionRunRequest
from automio.models.action_run_response import ActionRunResponse
from automio.models.action_run_status_response import ActionRunStatusResponse
from automio.models.action_type_enum import ActionTypeEnum
from automio.models.api_key_distribution_enum import ApiKeyDistributionEnum
from automio.models.apps_graphql_playground_app_parameter import AppsGraphqlPlaygroundAppParameter
from automio.models.apps_graphql_query400_response import AppsGraphqlQuery400Response
from automio.models.apps_graphql_query401_response import AppsGraphqlQuery401Response
from automio.models.apps_graphql_query404_response import AppsGraphqlQuery404Response
from automio.models.get_list200_response_inner import GetList200ResponseInner
from automio.models.graph_ql_request import GraphQLRequest
from automio.models.graph_ql_response import GraphQLResponse
from automio.models.graph_ql_response_errors_inner import GraphQLResponseErrorsInner
from automio.models.graph_ql_response_errors_inner_locations_inner import GraphQLResponseErrorsInnerLocationsInner
from automio.models.hmac_algorithm_enum import HmacAlgorithmEnum
from automio.models.hmac_key_format_enum import HmacKeyFormatEnum
from automio.models.integration import Integration
from automio.models.integration_category import IntegrationCategory
from automio.models.integration_type import IntegrationType
from automio.models.invoke_parallel_call200_response import InvokeParallelCall200Response
from automio.models.invoke_parallel_call_for_connection200_response import InvokeParallelCallForConnection200Response
from automio.models.invoke_parallel_call_for_connection_request import InvokeParallelCallForConnectionRequest
from automio.models.invoke_parallel_call_for_connection_request_data_list_inner import InvokeParallelCallForConnectionRequestDataListInner
from automio.models.invoke_parallel_call_request import InvokeParallelCallRequest
from automio.models.invoke_parallel_call_request_data_list_inner import InvokeParallelCallRequestDataListInner
from automio.models.invoke_parallel_call_request_data_list_inner_connection_id import InvokeParallelCallRequestDataListInnerConnectionId
from automio.models.invoke_parallel_call_request_data_list_inner_organization_id import InvokeParallelCallRequestDataListInnerOrganizationId
from automio.models.invoke_parallel_call_table_parameter import InvokeParallelCallTableParameter
from automio.models.language_enum import LanguageEnum
from automio.models.my_connections200_response_inner import MyConnections200ResponseInner
from automio.models.null_enum import NullEnum
from automio.models.paginated_integration_list import PaginatedIntegrationList
from automio.models.paginated_plugin_list import PaginatedPluginList
from automio.models.plugin import Plugin
from automio.models.plugin_integration import PluginIntegration
from automio.models.plugin_plugin_interface import PluginPluginInterface
from automio.models.status_enum import StatusEnum
from automio.models.table_connections200_response_value_inner import TableConnections200ResponseValueInner
from automio.models.table_connections400_response import TableConnections400Response
from automio.models.table_connections403_response import TableConnections403Response
from automio.models.table_connections404_response import TableConnections404Response
from automio.models.tag import Tag



# coding: utf-8

# flake8: noqa

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "0.1.2"

# import apis into sdk package
from automio.api.actions_api import ActionsApi
from automio.api.applications_tables_graph_ql_api import ApplicationsTablesGraphQLApi
from automio.api.applications_tables_tags_api import ApplicationsTablesTagsApi
from automio.api.integrations_api import IntegrationsApi
from automio.api.integrations_api_calls_api import IntegrationsAPICallsApi
from automio.api.integrations_email_service_api import IntegrationsEmailServiceApi
from automio.api.plugin_interfaces_plugins_api import PluginInterfacesPluginsApi
from automio.api.plugins_api import PluginsApi

# import ApiClient
from automio.api_response import ApiResponse
from automio.api_client import ApiClient
from automio.configuration import Configuration
from automio.exceptions import OpenApiException
from automio.exceptions import ApiTypeError
from automio.exceptions import ApiValueError
from automio.exceptions import ApiKeyError
from automio.exceptions import ApiAttributeError
from automio.exceptions import ApiException

# import models into sdk package
from automio.models.action import Action
from automio.models.action_run_request import ActionRunRequest
from automio.models.action_run_response import ActionRunResponse
from automio.models.action_run_status_response import ActionRunStatusResponse
from automio.models.action_type_enum import ActionTypeEnum
from automio.models.api_key_distribution_enum import ApiKeyDistributionEnum
from automio.models.apps_graphql_playground_app_parameter import AppsGraphqlPlaygroundAppParameter
from automio.models.apps_graphql_query400_response import AppsGraphqlQuery400Response
from automio.models.apps_graphql_query401_response import AppsGraphqlQuery401Response
from automio.models.apps_graphql_query404_response import AppsGraphqlQuery404Response
from automio.models.get_list200_response_inner import GetList200ResponseInner
from automio.models.graph_ql_request import GraphQLRequest
from automio.models.graph_ql_response import GraphQLResponse
from automio.models.graph_ql_response_errors_inner import GraphQLResponseErrorsInner
from automio.models.graph_ql_response_errors_inner_locations_inner import GraphQLResponseErrorsInnerLocationsInner
from automio.models.hmac_algorithm_enum import HmacAlgorithmEnum
from automio.models.hmac_key_format_enum import HmacKeyFormatEnum
from automio.models.integration import Integration
from automio.models.integration_category import IntegrationCategory
from automio.models.integration_type import IntegrationType
from automio.models.invoke_parallel_call200_response import InvokeParallelCall200Response
from automio.models.invoke_parallel_call_for_connection200_response import InvokeParallelCallForConnection200Response
from automio.models.invoke_parallel_call_for_connection_request import InvokeParallelCallForConnectionRequest
from automio.models.invoke_parallel_call_for_connection_request_data_list_inner import InvokeParallelCallForConnectionRequestDataListInner
from automio.models.invoke_parallel_call_request import InvokeParallelCallRequest
from automio.models.invoke_parallel_call_request_data_list_inner import InvokeParallelCallRequestDataListInner
from automio.models.invoke_parallel_call_request_data_list_inner_connection_id import InvokeParallelCallRequestDataListInnerConnectionId
from automio.models.invoke_parallel_call_request_data_list_inner_organization_id import InvokeParallelCallRequestDataListInnerOrganizationId
from automio.models.invoke_parallel_call_table_parameter import InvokeParallelCallTableParameter
from automio.models.language_enum import LanguageEnum
from automio.models.my_connections200_response_inner import MyConnections200ResponseInner
from automio.models.null_enum import NullEnum
from automio.models.paginated_integration_list import PaginatedIntegrationList
from automio.models.paginated_plugin_list import PaginatedPluginList
from automio.models.plugin import Plugin
from automio.models.plugin_integration import PluginIntegration
from automio.models.plugin_plugin_interface import PluginPluginInterface
from automio.models.status_enum import StatusEnum
from automio.models.table_connections200_response_value_inner import TableConnections200ResponseValueInner
from automio.models.table_connections400_response import TableConnections400Response
from automio.models.table_connections403_response import TableConnections403Response
from automio.models.table_connections404_response import TableConnections404Response
from automio.models.tag import Tag



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.plugin import Plugin

class TestPlugin(unittest.TestCase):
    """Plugin unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Plugin:
        """Test Plugin
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Plugin`
        """
        model = Plugin()
        if include_optional:
            return Plugin(
                id = 56,
                organization = 56,
                plugin_interface = robotnikai.models.plugin_plugin_interface.Plugin_plugin_interface(
                    id = 56, 
                    namespace = '', 
                    title = '', ),
                integration = robotnikai.models.plugin_integration.Plugin_integration(
                    name = '', 
                    icon = '', 
                    integration_id = '', ),
                implemented_capabilities = [
                    ''
                    ],
                is_enabled = True,
                capabilities = [
                    ''
                    ],
                is_visible = True,
                code = '',
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return Plugin(
        )
        """

    def testPlugin(self):
        """Test Plugin"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



import json
import logging
import os
import sys
import time
import uuid
from http.client import RemoteDisconnected
from dataclasses import dataclass
from typing import Any, Iterator, List, Literal, Optional, Tuple, TypedDict, Union

import requests
import automio
from urllib3.exceptions import ProtocolError
from dotenv import load_dotenv

try:
    import redis
except ImportError:
    redis = None

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[logging.StreamHandler()],
)
logger = logging.getLogger(__name__)

load_dotenv()

DEFAULT_HOST = "https://automio.com"

RETRY_DELAYS_SECONDS = [15, 60, 300]


class IntegrationAccessDeniedError(Exception):
    """Raised when backend denies integration call before reaching external API."""


def _is_user_action() -> bool:
    action_type = os.environ.get("ACTION_TYPE")
    return action_type is None or action_type == "user_action"


def _response_json_or_none(response):
    try:
        return response.json()
    except Exception:
        return None


def _is_internal_forbidden_response(response) -> bool:
    if response.status_code != 403:
        return False
    payload = _response_json_or_none(response)
    return isinstance(payload, dict) and "detail" in payload and "api_response" not in payload


def _api_exception_json_or_none(exc):
    body = getattr(exc, "body", None)
    if isinstance(body, dict):
        return body
    if not isinstance(body, str):
        return None
    try:
        return json.loads(body)
    except Exception:
        return None


def _is_internal_forbidden_api_exception(exc) -> bool:
    if getattr(exc, "status", None) != 403:
        return False
    payload = _api_exception_json_or_none(exc)
    return isinstance(payload, dict) and "detail" in payload and "api_response" not in payload


def _should_retry_response(response) -> bool:
    status_code = response.status_code
    if status_code == 403:
        # Retry only for upstream integration 403, not for internal CUA authorization 403.
        return not _is_internal_forbidden_response(response)
    return status_code == 401 or status_code >= 500


def _request_with_retry(request_func):
    """Execute a requests call with exponential backoff on auth/5xx/connection errors."""
    last_response = None
    for attempt in range(len(RETRY_DELAYS_SECONDS) + 1):
        try:
            response = request_func()
            last_response = response

            if _should_retry_response(response):
                if attempt < len(RETRY_DELAYS_SECONDS):
                    logger.warning(
                        f"Status code {response.status_code}. Retrying in {RETRY_DELAYS_SECONDS[attempt]} seconds..."
                    )
                    time.sleep(RETRY_DELAYS_SECONDS[attempt])
                    continue
                logger.error(
                    f"Status code {response.status_code}. No more retries left."
                )
            return response
        except (requests.exceptions.ConnectionError, RemoteDisconnected, ProtocolError):
            if attempt < len(RETRY_DELAYS_SECONDS):
                logger.warning(
                    f"Connection error, retrying in {RETRY_DELAYS_SECONDS[attempt]} seconds..."
                )
                time.sleep(RETRY_DELAYS_SECONDS[attempt])
                continue
            logger.error("Connection error, no more retries left.")
            raise
    return last_response


@dataclass
class ApiResponse:
    """Standardized API response object."""

    status_code: int
    text: str
    headers: dict
    url: str
    ok: bool


@dataclass
class ParallelCallResponse:
    """Response object for parallel API calls containing all response data and metadata."""

    api_data: dict
    api_response: ApiResponse
    organization_id: Optional[int] = None
    connection_id: Optional[int] = None


class Tag(TypedDict):
    collection_name: str
    color: str
    id: int
    name: str
    count: int
    organization_id: int
    description: Optional[str]
    createdAt: str
    updatedAt: str


def retrieve_app_id():
    """Retrieve APP_ID or ORG_SLUG/APP_SLUG"""
    env_app_id = os.environ.get("APP_ID")
    env_org_slug = os.environ.get("ORG_SLUG")
    env_app_slug = os.environ.get("APP_SLUG")
    if env_app_id:
        try:
            return int(env_app_id)
        except ValueError:
            return str(env_app_id)
    elif env_org_slug and env_app_slug:
        return f"{env_org_slug}/{env_app_slug}"


class IntegrationsApi(
    automio.IntegrationsApi,
    automio.IntegrationsAPICallsApi,
):
    def __init__(self, api_client=None):
        super().__init__(api_client)
        self.connection_id = os.environ.get("CONNECTION_ID", None)
        self._initial_connection_set = self.connection_id is not None
        if self.connection_id:
            logger.info(f"Using CONNECTION_ID: {self.connection_id}")

    def get_integration(self, integration_id, connection_id=None, *args, **kwargs):
        try:
            app_id = retrieve_app_id()
            extra_headers = kwargs.pop("_headers", {}) or {}
            if app_id is not None and "X-App-Id" not in extra_headers:
                extra_headers["X-App-Id"] = str(app_id)
            integration = super().get_integration(
                integration_id,
                connection_id=connection_id,
                _headers=extra_headers,
                *args,
                **kwargs,
            )
            if not integration.connected_accounts:
                print(f"Integration '{integration_id}' has no connected accounts. Please ensure you have set up a connection and activated it for this app.")
                raise SystemExit(1)
            return integration
        except automio.exceptions.NotFoundException:
            print(f"Integration '{integration_id}' does not exist.")
            raise SystemExit(1)

    def call(
        self,
        integration: automio.Integration,
        method: str = None,
        endpoint: str = None,
        connection_id=None,
        organization_id=None,
        table=None,  # In format app_{APP_ID}_{TABLE_NAME}
        *args,
        **kwargs,
    ) -> Tuple[dict, ApiResponse]:
        """Make a call to the integration API.

        Args:
            integration (automio.Integration): The integration to call.
            method (str, optional): The HTTP method to use. Defaults to None.
            endpoint (str, optional): The API endpoint to call. Defaults to None.
            connection_id (str, optional): The connection ID to use. Defaults to None.
            organization_id (str, optional): The organization ID to use. Defaults to None.
            table (str, optional): The table to use. Defaults to None.

        Raises:
            Exception: If the API call fails.
            e: If an error occurs.

        Returns:
            Tuple[dict, ApiResponse]: The API response data and metadata.
        """
        # Make the request directly to the call endpoint
        api_url = f'{os.getenv("API_BASE_URL", DEFAULT_HOST).rstrip("/")}/api/integrations/{integration.integration_id}/call/'
        # Support file uploads by switching to multipart when files are present
        files = kwargs.get("files")
        request_headers = {
            "Authorization": "App-Token {}".format(os.environ.get("APP_TOKEN")),
            "X-App-Id": str(retrieve_app_id()),
        }

        # Only set explicit JSON content-type when no files are sent (requests will set multipart boundary otherwise)
        if not files:
            request_headers["Content-Type"] = "application/json"

        payload_common = {
            "method": method,
            "endpoint": endpoint,
            "request_headers": kwargs.get("headers", {}),
            "request_params": kwargs.pop("params", {}),
        }

        if files:
            payload = {
                **payload_common,
                "data": kwargs.get("data", kwargs.get("json", {})),
            }
            response = _request_with_retry(
                lambda: requests.post(
                    api_url,
                    params={
                        "connection_id": connection_id,
                        "organization_id": organization_id,
                        "table": table,
                    },
                    data=payload,
                    files=files,
                    headers=request_headers,
                )
            )
        else:
            payload = {
                **payload_common,
                "data": kwargs.get("data", kwargs.get("json", {})),
            }
            response = _request_with_retry(
                lambda: requests.post(
                    api_url,
                    params={
                        "connection_id": connection_id,
                        "organization_id": organization_id,
                        "table": table,
                    },
                    json=payload,
                    headers=request_headers,
                )
            )
        if response.status_code == 402:
            if organization_id:
                logger.info(
                    f"Organization #{organization_id}: not enough credits to make the API call."
                )
            else:
                logger.info(
                    f"Your organization does not have enough credits to make the parallel API call."
                )
            if _is_user_action():
                sys.exit(0)
            return None
        if _is_internal_forbidden_response(response):
            payload = _response_json_or_none(response) or {}
            detail = payload.get("detail") or "Access denied by backend."
            raise IntegrationAccessDeniedError(detail)

        try:
            api_response = None
            response_data = response.json()
            if "api_response" in response_data:
                api_response = ApiResponse(
                    status_code=response_data["api_response"]["status_code"],
                    text=response_data["api_response"]["text"],
                    headers=response_data["api_response"]["headers"],
                    url=response_data["api_response"]["url"],
                    ok=response_data["api_response"]["ok"],
                )
        except json.JSONDecodeError:
            # If the response is not JSON, create a generic ApiResponse
            api_response = ApiResponse(
                status_code=response.status_code,
                text=response.text,
                headers=dict(response.headers),
                url=response.url,
                ok=response.ok,
            )

        try:
            if not response.ok:
                raise Exception(response.text)
            api_data = response_data.get("api_data", {})
            if not self._initial_connection_set and "connection_id" in response_data:
                self.connection_id = response_data["connection_id"]
            return api_data, api_response
        except Exception as e:
            if not api_response:
                logger.error(f"Error in API call: {str(e)}")
                raise e
            return None, api_response

    def _format_responses(self, responses):
        """Format API responses into ParallelCallResponse objects.

        Args:
            responses: List of response dictionaries from the API

        Returns:
            List[ParallelCallResponse]: List of formatted response objects
        """
        full_response = []
        for r in responses:
            if "api_response" in r:
                api_response = ApiResponse(
                    status_code=r["api_response"]["status_code"],
                    text=r["api_response"]["text"],
                    headers=r["api_response"]["headers"],
                    url=r["api_response"]["url"],
                    ok=r["api_response"]["ok"],
                )
            else:
                api_response = None

            try:
                organization_id = int(r.get("organization_id"))
            except (TypeError, ValueError):
                organization_id = None

            try:
                connection_id = int(r.get("connection_id"))
            except (TypeError, ValueError):
                connection_id = None

            try:
                api_data = r.get("api_data", {})
                response_obj = ParallelCallResponse(
                    api_data=api_data,
                    api_response=api_response,
                    organization_id=organization_id,
                    connection_id=connection_id,
                )
                full_response.append(response_obj)
            except Exception as e:
                logger.error(f"Error processing response: {str(e)}")
                error_response_obj = ParallelCallResponse(
                    api_data=None,
                    api_response=api_response,
                    organization_id=organization_id,
                    connection_id=connection_id,
                )
                full_response.append(error_response_obj)
        return full_response

    def parallel_call_stream(
        self,
        integration: automio.Integration,
        method,
        endpoint,
        data_list,
        connection_id=None,
        *args,
        **kwargs,
    ):
        """
        Generator method for streaming parallel calls.
        Yields individual responses as they become available.

        Yields:
            dict: Individual response data with keys: index, response, completed, total
        """

        payload = {
            "method": method,
            "endpoint": endpoint,
            "data_list": data_list,
            "request_headers": kwargs.get("headers", {}),
            "request_params": kwargs.pop("params", {}),
            "streaming": True,
        }

        # Make the request directly to the parallel_call endpoint
        api_url = f'{os.getenv("API_BASE_URL", DEFAULT_HOST).rstrip("/")}/api/integrations/{integration.integration_id}/parallel_call/'

        response = _request_with_retry(
            lambda: requests.post(
                api_url,
                params={"connection_id": connection_id},
                json=payload,
                headers={
                    "Authorization": "App-Token {}".format(os.environ.get("APP_TOKEN")),
                    "Content-Type": "application/json",
                    "X-App-Id": str(retrieve_app_id()),
                },
                stream=True,
                timeout=300,  # Longer timeout for streaming
            )
        )
        if response.status_code == 402:
            logger.info(f"Not enough credits to make the parallel API call.")
            if _is_user_action():
                sys.exit(0)
            return None

        if _is_internal_forbidden_response(response):
            payload = _response_json_or_none(response) or {}
            detail = payload.get("detail") or "Access denied by backend."
            raise IntegrationAccessDeniedError(detail)

        if not response.status_code in [200, 201, 202, 204]:
            raise Exception(response.text)

        # Parse streaming response and yield individual results
        for line in response.iter_lines(decode_unicode=True):
            if line.startswith("data: "):
                try:
                    data = json.loads(line[6:])  # Remove 'data: ' prefix
                    # Set connection_id when available
                    if not self._initial_connection_set and data.get("connection_id"):
                        self.connection_id = data["connection_id"]

                    if data.get("completed") is True:
                        # Final completion message - yield and break
                        yield {
                            "final": True,
                            "total_processed": data.get("total_processed"),
                        }
                        break
                    else:
                        api_data = data["response"]["api_data"]
                        api_response = ApiResponse(
                            status_code=data["response"]["api_response"]["status_code"],
                            text=data["response"]["api_response"]["text"],
                            headers=data["response"]["api_response"]["headers"],
                            url=data["response"]["api_response"]["url"],
                            ok=data["response"]["api_response"]["ok"],
                        )
                        # Individual response - yield it
                        yield {
                            "data": api_data,
                            "response": api_response,
                            "index": data.get("index"),
                            "total": data.get("total"),
                        }
                except json.JSONDecodeError:
                    continue

    def parallel_call_for_connection(
        self,
        integration: automio.Integration,
        method,
        endpoint,
        data_list,
        connection_id,
        organization_id=None,
        table=None,  # In format app_{APP_ID}_{TABLE_NAME}
        *args,
        **kwargs,
    ) -> List[ParallelCallResponse]:
        try:
            response = self.invoke_parallel_call_for_connection_with_http_info(
                x_app_id=retrieve_app_id(),
                integration_id=integration.integration_id,
                invoke_parallel_call_for_connection_request=automio.InvokeParallelCallForConnectionRequest(
                    method=method,
                    endpoint=endpoint,
                    data_list=data_list,
                    request_headers=kwargs.get("headers", {}),
                    streaming=False,
                ),
                connection_id=connection_id,
                organization_id=organization_id,
                table=table,
            )
        except automio.exceptions.ApiException as exc:
            if _is_internal_forbidden_api_exception(exc):
                payload = _api_exception_json_or_none(exc) or {}
                detail = payload.get("detail") or "Access denied by backend."
                raise IntegrationAccessDeniedError(detail) from exc
            raise
        if response.status_code == 402:
            if organization_id:
                logger.info(
                    f"Organization #{organization_id}: not enough credits to make the parallel API call."
                )
            else:
                logger.info(
                    f"Your organization does not have enough credits to make the parallel API call."
                )
            if _is_user_action():
                sys.exit(0)
            return None

        full_response = self._format_responses(response.data.responses)

        if not self._initial_connection_set and response.data.connection_id:
            self.connection_id = response.data.connection_id
        return full_response

    def parallel_call(
        self,
        integration: automio.Integration,
        method,
        endpoint,
        data_list,
        table=None,  # In format app_{APP_ID}_{TABLE_NAME}
        *args,
        **kwargs,
    ) -> List[ParallelCallResponse]:
        """
        Execute parallel API calls using multiple connections with per-object connection handling.

        This method processes a list of data objects, each containing its own connection_id and
        organization_id, and makes parallel API requests to the specified endpoint. Each request
        is authenticated using the connection specified in the data object.

        Args:
            integration (automio.Integration): The integration object containing integration_id
                used to identify which API integration to call.
            method (str): HTTP method for the API call (e.g., 'GET', 'POST', 'PUT', 'DELETE').
            endpoint (str): The API endpoint path to call (relative to the integration's base URL).
            data_list (List[dict]): List of data objects, each must contain:
                - connection_id (int): ID of the connection to use for this request
                - organization_id (int): ID of the organization associated with the connection
                - data (dict, optional): Request body data for this specific call
                - params (dict, optional): Query parameters for this specific call
                - url_params (dict, optional): URL path parameters for this specific call
            table (str, optional): Table identifier in format app_{APP_ID}_{TABLE_NAME}.
                Used for data persistence and tracking. Defaults to None.
            **kwargs: Additional keyword arguments:
                - headers (dict): Request headers to include in all calls

        Returns:
            List[ParallelCallResponse]: List of response objects, each containing:
                - api_data (dict): API response data from the external service
                - api_response (ApiResponse): HTTP response metadata (status, headers, url, etc.)
                - organization_id (int|None): Organization ID used for this request
                - connection_id (int|None): Connection ID used for this request

        Raises:
            Exception: If the API request fails or if required fields are missing from data_list items.

        Example:
            >>> data_list = [
            ...     {
            ...         "connection_id": 123,
            ...         "organization_id": 456,
            ...         "data": {"name": "John", "age": 30},
            ...         "params": {"include": "profile"}
            ...     },
            ...     {
            ...         "connection_id": 789,
            ...         "organization_id": 101,
            ...         "data": {"name": "Jane", "age": 25}
            ...     }
            ... ]
            >>> results = api.integrations.parallel_call(
            ...     integration=my_integration,
            ...     method="POST",
            ...     endpoint="/users",
            ...     data_list=data_list,
            ...     table="app_123_users"
            ... )
            >>> for response in results:
            ...     print(f"Org {response.organization_id}, Conn {response.connection_id}")
            ...     print(f"Data: {response.api_data}")
            ...     print(f"Status: {response.api_response.status_code}")
        """
        try:
            response = self.invoke_parallel_call_with_http_info(
                x_app_id=retrieve_app_id(),
                integration_id=integration.integration_id,
                invoke_parallel_call_request=automio.InvokeParallelCallRequest(
                    method=method,
                    endpoint=endpoint,
                    data_list=[
                        automio.InvokeParallelCallRequestDataListInner(
                            connection_id=automio.InvokeParallelCallRequestDataListInnerConnectionId(
                                str(obj["connection_id"])
                            ),
                            organization_id=automio.InvokeParallelCallRequestDataListInnerOrganizationId(
                                str(obj["organization_id"])
                            ),
                            params=obj.get("params", {}),
                            data=obj.get("data", {}),
                            url_params=obj.get("url_params", {}),
                        )
                        for obj in data_list
                    ],
                    request_headers=kwargs.get("headers", {}),
                    streaming=False,
                ),
                table=table,
            )
        except automio.exceptions.ApiException as exc:
            if _is_internal_forbidden_api_exception(exc):
                payload = _api_exception_json_or_none(exc) or {}
                detail = payload.get("detail") or "Access denied by backend."
                raise IntegrationAccessDeniedError(detail) from exc
            raise

        if response.status_code == 402:
            logger.info(f"Not enough credits to make the parallel API call.")
            if _is_user_action():
                sys.exit(0)
            return None

        full_response = self._format_responses(response.data.responses)

        return full_response

    def all_connections(
        self, integration: automio.Integration, table_name: str, app_id=None
    ) -> dict:
        """Get all connections for a specific integration and table.

        Args:
            integration (automio.Integration): The integration to get connections for.
            table_name (str): The name of the table to get connections for.
            app_id (str, optional): The app ID to get connections for. If not provided, it will be retrieved from the environment.

        Returns:
            dict: A dictionary mapping organization IDs to lists of connections.
        """
        return self.table_connections(
            integration_id=integration.integration_id,
            table=table_name,
            app_id=app_id if app_id else retrieve_app_id(),
        )


class ActionsApi(automio.ActionsApi):
    def __init__(self, api_client=None):
        super().__init__(api_client)

    def _auth_headers(self, include_content_type: bool = False) -> dict:
        headers = {
            "Authorization": self.api_client.configuration.api_key.get("knoxApiToken"),
        }
        if include_content_type:
            headers["Content-Type"] = "application/json"
        return headers

    def list(self) -> List[automio.Action]:
        app_id = retrieve_app_id()
        if not app_id:
            raise ValueError("APP_ID variable is not set")

        response = _request_with_retry(
            lambda: requests.get(
                f'{os.getenv("API_BASE_URL", DEFAULT_HOST).rstrip("/")}/api/actions/',
                params={"app": app_id},
                headers=self._auth_headers(),
            )
        )

        if not response.ok:
            raise Exception(response.text)

        payload = response.json() if response.text else []
        return [automio.Action.from_dict(item) for item in payload]

    def run(
        self,
        action_id: Union[int, str],
        selected_rows: Optional[List[str]] = None,
        selected_all: Optional[bool] = None,
        filter_params: Optional[dict] = None,
        user_input: Optional[dict] = None,
        connection_id: Optional[str] = None,
        language: Optional[str] = None,
    ) -> automio.ActionRunResponse:
        body = {}
        if selected_rows is not None:
            body["selected_rows"] = selected_rows
        if selected_all is not None:
            body["selected_all"] = selected_all
        if filter_params is not None:
            body["filter_params"] = filter_params
        if user_input is not None:
            body["user_input"] = user_input
        if connection_id is not None:
            body["connection_id"] = connection_id
        if language is not None:
            body["language"] = language

        action_run_request = (
            automio.ActionRunRequest.from_dict(body) if body else None
        )
        try:
            return self.actions_run_create(id=int(action_id), action_run_request=action_run_request)
        except automio.exceptions.ApiException as exc:
            if exc.status == 402:
                payload = _api_exception_json_or_none(exc) or {}
                detail = payload.get("detail") or "Not enough credits to run the action."
                logger.info(detail)
                if _is_user_action():
                    sys.exit(0)
                return None
            raise

    def run_status(self, run_id: Union[int, str]) -> automio.ActionRunStatusResponse:
        return self.actions_runs_status_retrieve(id=int(run_id))

    def runStatus(self, run_id: Union[int, str]) -> automio.ActionRunStatusResponse:
        return self.run_status(run_id)

    def stream_logs(
        self,
        task_id: str,
        skip_existing: bool = False,
        timeout: int = 300,
    ) -> Iterator[str]:
        if not task_id:
            raise ValueError("stream_logs: 'task_id' is required")

        app_id = retrieve_app_id()
        if not app_id:
            raise ValueError("APP_ID variable is not set")

        params = {"task_id": task_id, "app": str(app_id)}
        if skip_existing:
            params["skip_existing"] = "1"

        response = _request_with_retry(
            lambda: requests.get(
                f'{os.getenv("API_BASE_URL", DEFAULT_HOST).rstrip("/")}/api/actions/run-code-stream/',
                params=params,
                headers={
                    **self._auth_headers(),
                    "Accept": "text/event-stream",
                    "Cache-Control": "no-cache",
                },
                stream=True,
                timeout=timeout,
            )
        )

        if not response.ok:
            raise Exception(
                f"SSE request failed ({response.status_code}): {response.text}"
            )

        try:
            for line in response.iter_lines(decode_unicode=True):
                if line and line.startswith("data:"):
                    payload = line[5:].strip()
                    if payload:
                        yield payload
        finally:
            response.close()

    def streamLogs(
        self,
        task_id: str,
        skip_existing: bool = False,
        timeout: int = 300,
    ) -> Iterator[str]:
        yield from self.stream_logs(
            task_id=task_id,
            skip_existing=skip_existing,
            timeout=timeout,
        )


class CacheApi:
    def __init__(self, cache_config=None):
        # Use shared cache configuration
        self.cache_service_url = cache_config["cache_service_url"]
        self.task_id = cache_config["task_id"]
        self.session = cache_config["session"]
        self.redis_client = cache_config["redis_client"]

    def _get_redis_key(self, key: str) -> str:
        """Generate Redis key with proper namespacing - to be overridden by subclasses"""
        raise NotImplementedError("Subclasses must implement _get_redis_key")

    def _get_base_params(self) -> dict:
        """Get base parameters for HTTP cache service - to be overridden by subclasses"""
        raise NotImplementedError("Subclasses must implement _get_base_params")

    def _get_set_payload(self, key: str, value: Any, ttl: Optional[int] = None) -> dict:
        """Get payload for HTTP cache service set operation - to be overridden by subclasses"""
        return {"key": key, "value": value, "ttl": ttl}

    def get(self, key: str) -> Any:
        """Get cached value by key"""
        if self.cache_service_url:
            # Use HTTP cache service
            try:
                response = self.session.get(
                    f"{self.cache_service_url}/get/{key}/",
                    params=self._get_base_params(),
                )
                return response.json()
            except Exception as e:
                logger.error(f"Error getting cache for key {key}: {str(e)}")
                return None
        else:
            # Use Redis directly
            try:
                redis_key = self._get_redis_key(key)
                value = self.redis_client.get(redis_key)
                if value is None:
                    return None
                return {
                    "key": key,
                    "value": value,
                    "exists": True,
                    "ttl": self.redis_client.ttl(redis_key),
                }
            except Exception as e:
                logger.error(f"Error getting cache for key {key}: {str(e)}")
                return None

    def set(self, key: str, value: Any, ttl: Optional[int] = None) -> bool:
        """Set cached value by key"""
        if self.cache_service_url:
            # Use HTTP cache service
            try:
                response = self.session.post(
                    f"{self.cache_service_url}/set/",
                    params=self._get_base_params(),
                    json=self._get_set_payload(key, value, ttl),
                )
                return response.json()
            except Exception as e:
                logger.error(f"Error setting cache for key {key}: {str(e)}")
                return False
        else:
            # Use Redis directly
            try:
                redis_key = self._get_redis_key(key)
                serialized_value = json.dumps(value)
                if ttl:
                    self.redis_client.setex(redis_key, ttl, serialized_value)
                else:
                    self.redis_client.set(redis_key, serialized_value)
                return {
                    "success": True,
                    "key": key,
                    "message": "Cache set successfully",
                    "ttl": ttl,
                }
            except Exception as e:
                logger.error(f"Error setting cache for key {key}: {str(e)}")
                return False

    def delete(self, key: str) -> bool:
        """Delete cached value by key"""
        if self.cache_service_url:
            # Use HTTP cache service
            try:
                response = self.session.delete(
                    f"{self.cache_service_url}/delete/{key}/",
                    params=self._get_base_params(),
                )
                return response.json()
            except Exception as e:
                logger.error(f"Error deleting cache for key {key}: {str(e)}")
                return False
        else:
            # Use Redis directly
            try:
                redis_key = self._get_redis_key(key)
                deleted_count = self.redis_client.delete(redis_key)
                message = (
                    "Key deleted successfully" if deleted_count > 0 else "Key not found"
                )
                return {
                    "success": True,
                    "message": message,
                    "key": key,
                    "deleted": deleted_count > 0,
                }
            except Exception as e:
                logger.error(f"Error deleting cache for key {key}: {str(e)}")
                return False


class AppCacheApi(CacheApi):
    def __init__(self, cache_config=None):
        super().__init__(cache_config)
        self.action_id = os.environ.get("ACTION_ID")
        self.app_id = retrieve_app_id()

    def _get_redis_key(self, key: str) -> str:
        """Generate Redis key with proper namespacing"""
        return f"app:{self.app_id}:{key}"

    def _get_base_params(self) -> dict:
        """Get base parameters for HTTP cache service"""
        return {
            "task_id": self.task_id,
            "action_id": self.action_id,
            "cache_type": "app",
        }

    def _get_set_payload(self, key: str, value: Any, ttl: Optional[int] = None) -> dict:
        """Get payload for HTTP cache service set operation"""
        return {
            "action": self.action_id,
            "key": key,
            "value": value,
            "ttl": ttl,
        }


class UserCacheApi(CacheApi):
    def __init__(self, cache_config=None):
        super().__init__(cache_config)

    def _get_redis_key(self, key: str) -> str:
        """Generate Redis key with proper namespacing"""
        return f"org:{self.task_id}:{key}"

    def _get_base_params(self) -> dict:
        """Get base parameters for HTTP cache service"""
        return {
            "task_id": self.task_id,
            "cache_type": "org",
        }


class GraphqlException(Exception):
    """Custom exception for GraphQL errors."""


class TablesApi(
    automio.ApplicationsTablesGraphQLApi, automio.ApplicationsTablesTagsApi
):
    def __init__(
        self, client: automio.ApiClient, integrations_api: "IntegrationsApi"
    ):
        super().__init__(client)
        self.client = client
        self.integrations_api = integrations_api

    def graphql(
        self,
        table: str,
        query: str = None,
        variables: dict = None,
        graph_ql_request: automio.GraphQLRequest = None,
    ) -> Any:
        """Make a GraphQL query to a specific table.

        Args:
            table (str): The name of the table to query.
            query (str, optional): The GraphQL query string. Defaults to None.
            variables (dict, optional): The variables for the GraphQL query. Defaults to None.
            graph_ql_request (automio.GraphQLRequest, optional): The GraphQL request object. Defaults to None.

        Raises:
            ValueError: If the table name is not provided.
            ValueError: If the query is not provided.
            GraphqlException: If the GraphQL query fails.
            e: If an error occurs.

        Returns:
            Any: The response from the GraphQL query.
        """
        app_id = retrieve_app_id()

        if not app_id:
            raise ValueError("APP_ID variable is not set")

        if not graph_ql_request:
            if not query:
                raise ValueError(
                    "Either 'query' or 'graph_ql_request' must be provided"
                )
            graph_ql_request = automio.GraphQLRequest(
                query=query, variables=variables
            )

        # Automatically inject connection_id from integrations API if available
        if self.integrations_api.connection_id:
            if graph_ql_request.variables is None:
                graph_ql_request.variables = {}

            # Only inject if connection_id is not already present
            first_variable_key = next(iter(graph_ql_request.variables), None)
            try:
                if (
                    first_variable_key
                    and "connection_id"
                    not in graph_ql_request.variables[first_variable_key]
                ):
                    for object in graph_ql_request.variables[first_variable_key]:
                        if isinstance(object, dict):
                            object["connection_id"] = (
                                self.integrations_api.connection_id
                            )

                    logger.info(
                        f"Auto-injected connection_id ({self.integrations_api.connection_id}) to graphql variables"
                    )
            except Exception as e:
                logger.debug(
                    f"Error injecting connection_id to graphql variables: {str(e)}"
                )

        try:
            response = self.apps_graphql_query(
                app=app_id,
                table=table,
                graph_ql_request=graph_ql_request,  # type: ignore
            )
            if response.errors and len(response.errors) > 0:
                for i, error in enumerate(response.errors):
                    # Log errors as detailed as possible
                    logger.error(f"GraphQL Error #{i + 1}: {error.message}")
                    if getattr(error, "extensions", None):
                        logger.error(f"Extensions: {error.extensions}")
                    if getattr(error, "locations", None):
                        logger.error(f"Locations: {error.locations}")
                    if getattr(error, "path", None):
                        logger.error(f"Path: {error.path}")

                raise GraphqlException("GraphQL errors occurred", response.errors)
            return response
        except automio.exceptions.ApiException as exc:
            if exc.status == 402:
                logger.info("Not enough credits to make the GraphQL query.")
                if _is_user_action():
                    sys.exit(0)
                return None
            raise
        except Exception as e:
            raise e

    def create_tag(self, table_name: str, tag_name: str) -> Tag:
        """
        Create a new tag for a specific table.

        Args:
            table_name (str): The name of the table to create the tag for.
            tag_name (str): The name of the tag to create.
        """
        from automio.models import Tag as TagModel

        response = self.add_tag_with_http_info(
            app_id=str(os.environ.get("APP_ID")),
            table_name=table_name,
            tag=TagModel(name=tag_name),
        )
        if response.status_code not in [200, 201]:
            raise Exception(
                f"Error creating tag: {response.status_code} - {response.raw_data}"
            )
        return Tag(json.loads(response.raw_data)["tag"])


class Task(CacheApi):
    def __init__(self, cache_config=None):
        super().__init__(cache_config)

    def set_progress(
        self,
        progress: int,
        info: str,
        status: Literal["pending", "completed", "failed"] = "pending",
    ):
        """
        Set the progress of the current task.

        Parameters:
            progress (int): Progress percentage (0-100)
            info (str): Additional information about the task
            status (str): Status of the task (e.g., 'pending', 'completed', 'failed')
        """
        if not self.task_id:
            logger.error("TASK_ID variable is not set")
            return

        if self.cache_service_url:
            # Use HTTP cache service
            try:
                response = self.session.post(
                    f"{self.cache_service_url}/set-progress/{self.task_id}/",
                    json={
                        "progress": progress,
                        "info": info,
                        "status": status,
                    },
                )
                return response.json()
            except Exception as e:
                logger.error(
                    f"Error setting progress for task {self.task_id}: {str(e)}"
                )
                return None
        else:
            # Use Redis directly - store progress info
            try:
                progress_key = f"task:progress:{self.task_id}"
                progress_data = {
                    "progress": progress,
                    "info": info,
                    "status": status,
                    "updated_at": str(uuid.uuid4()),  # Using uuid as simple timestamp
                }
                self.redis_client.set(progress_key, json.dumps(progress_data))
                logger.info(f"Task {self.task_id} progress set to {progress}%: {info}")
                return {
                    "success": True,
                    "message": "Progress updated successfully",
                    "task_id": self.task_id,
                    "progress": int(progress),
                    "info": info,
                    "status": status,
                }
            except Exception as e:
                logger.error(
                    f"Error setting progress for task {self.task_id}: {str(e)}"
                )
                return None


class PluginsApi(automio.PluginInterfacesPluginsApi, automio.PluginsApi):
    def __init__(self, api_client=None):
        super().__init__(api_client)

    def invoke(
        self,
        plugin: str,
        integration: str | automio.Integration,
        connection_ids: List[Union[str, int]] = [],
        **payload,
    ) -> Any:
        """Invoke a plugin capability.

        Args:
            plugin (str): The name of the plugin to invoke.
            integration (str | automio.Integration): The name of the integration to use.
            connection_ids (List[Union[str, int]], optional): The connection IDs to use. Defaults to [].

        Raises:
            ImportError: If the plugin module cannot be imported.
            AttributeError: If the plugin capability is not found.

        Returns:
            Any: The result of the plugin invocation.
        """
        namespace, capability = plugin.split(".")
        logger.info(
            f"Invoking plugin: {namespace}.{capability} with integration: {integration}"
        )

        if isinstance(integration, automio.Integration):
            integration_id = integration.integration_id
        else:
            integration_id = integration

        try:
            # Import plugin module
            module_name = f"plugins.{namespace}_{integration_id}"
            module = __import__(module_name, fromlist=[capability])

            # Get the capability function from the module
            capability_func = getattr(module, capability)

            # Invoke the capability function
            return capability_func(connection_ids=connection_ids, **payload)
        except ImportError as e:
            logger.error(f"Failed to import module {module_name}: {e}")
            raise ImportError(f"Plugin module '{module_name}' not found")
        except AttributeError as e:
            logger.error(
                f"Capability '{capability}' not found in module {module_name}: {e}"
            )
            raise AttributeError(
                f"Capability '{capability}' not implemented in plugin '{namespace}_{integration_id}'"
            )
        except NotImplementedError as e:
            logger.error(
                f"Capability '{capability}' in plugin '{namespace}_{integration_id}' is not implemented"
            )
        except Exception as e:
            logger.error(f"Error invoking capability '{capability}': {e}")
            raise


class API:
    def __init__(self):
        # Initialize shared cache configuration once
        try:
            self._cache_config = self._init_cache_config()
        except Exception as e:
            logger.error(f"Error initializing cache configuration: {str(e)}")
            self._cache_config = None

        self._client = automio.ApiClient(self.configuration)
        self.actions = ActionsApi(self._client)
        self.integrations = IntegrationsApi(self._client)
        self.plugins = PluginsApi(self._client)
        self.tables = TablesApi(self._client, self.integrations)
        if self._cache_config:
            self.app_cache = AppCacheApi(self._cache_config)
            self.user_cache = UserCacheApi(self._cache_config)
            self.task = Task(self._cache_config)

    def _init_cache_config(self):
        """Initialize cache configuration once and share across all cache instances"""
        cache_service_url = os.getenv("CACHE_SERVICE_URL")
        task_id = str(os.environ.get("TASK_ID", uuid.uuid4()))

        if cache_service_url:
            # Use HTTP cache service
            session = requests.Session()
            cache_service_token = os.environ.get("CACHE_SERVICE_TOKEN")
            session.headers.update(
                {
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {cache_service_token}",
                }
            )
            redis_client = None
            logger.info(
                f"Using HTTP cache service at {cache_service_url} with task ID {task_id}"
            )
        else:
            session = None
            # Use direct Redis connection
            if redis is None:
                raise ImportError(
                    "Redis library is not installed. Please install it with: pip install redis"
                )

            redis_host = os.getenv("REDIS_HOST", "localhost")
            redis_port = int(os.getenv("REDIS_PORT", 6379))
            redis_db = int(os.getenv("REDIS_DB", 0))
            redis_password = os.getenv("REDIS_PASSWORD")

            try:
                redis_client = redis.Redis(
                    host=redis_host,
                    port=redis_port,
                    db=redis_db,
                    password=redis_password,
                    decode_responses=True,
                )
                # Test the connection
                redis_client.ping()
                logger.info(
                    f"Using Redis cache at {redis_host}:{redis_port}/{redis_db} with task ID {task_id}"
                )
            except redis.ConnectionError as e:
                raise ConnectionError(
                    f"Failed to connect to Redis at {redis_host}:{redis_port}/{redis_db}. "
                    f"Please ensure Redis is running and accessible. Error: {str(e)}"
                )
            except redis.AuthenticationError as e:
                raise ConnectionError(
                    f"Failed to authenticate with Redis at {redis_host}:{redis_port}/{redis_db}. "
                    f"Please check your REDIS_PASSWORD. Error: {str(e)}"
                )
            except Exception as e:
                raise ConnectionError(
                    f"Failed to initialize Redis connection at {redis_host}:{redis_port}/{redis_db}. "
                    f"Error: {str(e)}"
                )

        return {
            "cache_service_url": cache_service_url,
            "task_id": task_id,
            "session": session,
            "redis_client": redis_client,
        }

    @property
    def configuration(self):
        configuration = automio.Configuration()
        configuration.host = os.getenv("API_BASE_URL", DEFAULT_HOST)
        configuration.api_key["knoxApiToken"] = "App-Token {}".format(
            os.environ.get("APP_TOKEN")
        )
        return configuration

    def close(self):
        self._client.close()

    def notify_me(self, subject: str, text: Optional[str], html: Optional[str] = None):
        """
        Send a notification to the user.

        Parameters:
            subject (str): Subject of the notification
            text (Optional[str]): Text content of the notification
            html (Optional[str]): HTML content of the notification
        """
        if not text and not html:
            raise ValueError("At least one of 'text' or 'html' must be provided")

        response = _request_with_retry(
            lambda: requests.post(
                f"{self.configuration.host}/api/integrations/mailer/notify-me/",
                json={
                    "subject": subject,
                    "text": text,
                    "html": html,
                },
                headers={
                    "Authorization": self.configuration.api_key["knoxApiToken"],
                    "Content-Type": "application/json",
                },
            )
        )
        return response.json() if response.ok else response.text



# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ActionTypeEnum(str, Enum):
    """
    * `app_action` - App Action * `user_action` - User Action * `table_action` - Table Action * `trigger_action` - Trigger Action * `webhook_action` - Webhook Action
    """

    """
    allowed enum values
    """
    APP_ACTION = 'app_action'
    USER_ACTION = 'user_action'
    TABLE_ACTION = 'table_action'
    TRIGGER_ACTION = 'trigger_action'
    WEBHOOK_ACTION = 'webhook_action'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ActionTypeEnum from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.paginated_plugin_list import PaginatedPluginList

class TestPaginatedPluginList(unittest.TestCase):
    """PaginatedPluginList unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaginatedPluginList:
        """Test PaginatedPluginList
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaginatedPluginList`
        """
        model = PaginatedPluginList()
        if include_optional:
            return PaginatedPluginList(
                count = 123,
                next = 'http://api.example.org/accounts/?page=4',
                previous = 'http://api.example.org/accounts/?page=2',
                results = [
                    robotnikai.models.plugin.Plugin(
                        id = 56, 
                        organization = 56, 
                        plugin_interface = robotnikai.models.plugin_plugin_interface.Plugin_plugin_interface(
                            id = 56, 
                            namespace = '', 
                            title = '', ), 
                        integration = robotnikai.models.plugin_integration.Plugin_integration(
                            name = '', 
                            icon = '', 
                            integration_id = '', ), 
                        implemented_capabilities = [
                            ''
                            ], 
                        is_enabled = True, 
                        capabilities = [
                            ''
                            ], 
                        is_visible = True, 
                        code = '', 
                        created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                    ]
            )
        else:
            return PaginatedPluginList(
                count = 123,
                results = [
                    robotnikai.models.plugin.Plugin(
                        id = 56, 
                        organization = 56, 
                        plugin_interface = robotnikai.models.plugin_plugin_interface.Plugin_plugin_interface(
                            id = 56, 
                            namespace = '', 
                            title = '', ), 
                        integration = robotnikai.models.plugin_integration.Plugin_integration(
                            name = '', 
                            icon = '', 
                            integration_id = '', ), 
                        implemented_capabilities = [
                            ''
                            ], 
                        is_enabled = True, 
                        capabilities = [
                            ''
                            ], 
                        is_visible = True, 
                        code = '', 
                        created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                        updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), )
                    ],
        )
        """

    def testPaginatedPluginList(self):
        """Test PaginatedPluginList"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.invoke_parallel_call_for_connection_request import InvokeParallelCallForConnectionRequest

class TestInvokeParallelCallForConnectionRequest(unittest.TestCase):
    """InvokeParallelCallForConnectionRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> InvokeParallelCallForConnectionRequest:
        """Test InvokeParallelCallForConnectionRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `InvokeParallelCallForConnectionRequest`
        """
        model = InvokeParallelCallForConnectionRequest()
        if include_optional:
            return InvokeParallelCallForConnectionRequest(
                method = 'GET',
                endpoint = '',
                data_list = [
                    robotnikai.models.invoke_parallel_call_for_connection_request_data_list_inner.invoke_parallel_call_for_connection_request_data_list_inner(
                        params = robotnikai.models.params.params(), 
                        data = robotnikai.models.data.data(), 
                        url_params = robotnikai.models.url_params.url_params(), )
                    ],
                request_headers = None,
                streaming = True
            )
        else:
            return InvokeParallelCallForConnectionRequest(
                method = 'GET',
                data_list = [
                    robotnikai.models.invoke_parallel_call_for_connection_request_data_list_inner.invoke_parallel_call_for_connection_request_data_list_inner(
                        params = robotnikai.models.params.params(), 
                        data = robotnikai.models.data.data(), 
                        url_params = robotnikai.models.url_params.url_params(), )
                    ],
                request_headers = None,
        )
        """

    def testInvokeParallelCallForConnectionRequest(self):
        """Test InvokeParallelCallForConnectionRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.api.applications_tables_graph_ql_api import ApplicationsTablesGraphQLApi


class TestApplicationsTablesGraphQLApi(unittest.TestCase):
    """ApplicationsTablesGraphQLApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ApplicationsTablesGraphQLApi()

    def tearDown(self) -> None:
        pass

    def test_apps_graphql_playground(self) -> None:
        """Test case for apps_graphql_playground

        GraphQL Playground Interface
        """
        pass

    def test_apps_graphql_query(self) -> None:
        """Test case for apps_graphql_query

        Execute GraphQL Query/Mutation
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.plugin_integration import PluginIntegration

class TestPluginIntegration(unittest.TestCase):
    """PluginIntegration unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PluginIntegration:
        """Test PluginIntegration
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PluginIntegration`
        """
        model = PluginIntegration()
        if include_optional:
            return PluginIntegration(
                name = '',
                icon = '',
                integration_id = ''
            )
        else:
            return PluginIntegration(
        )
        """

    def testPluginIntegration(self):
        """Test PluginIntegration"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.api.integrations_email_service_api import IntegrationsEmailServiceApi


class TestIntegrationsEmailServiceApi(unittest.TestCase):
    """IntegrationsEmailServiceApi unit test stubs"""

    def setUp(self) -> None:
        self.api = IntegrationsEmailServiceApi()

    def tearDown(self) -> None:
        pass

    def test_integrations_mailer_notify_me_create(self) -> None:
        """Test case for integrations_mailer_notify_me_create

        Send notification email
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from automio.models.plugin_integration import PluginIntegration
from automio.models.plugin_plugin_interface import PluginPluginInterface
from typing import Optional, Set
from typing_extensions import Self

class Plugin(BaseModel):
    """
    Plugin
    """ # noqa: E501
    id: Optional[StrictInt] = None
    organization: Optional[StrictInt] = None
    plugin_interface: Optional[PluginPluginInterface] = None
    integration: Optional[PluginIntegration] = None
    implemented_capabilities: Optional[List[StrictStr]] = None
    is_enabled: Optional[StrictBool] = None
    is_official: Optional[StrictBool] = None
    is_mine: Optional[StrictBool] = None
    organization_slug: Optional[StrictStr] = None
    capabilities: Optional[List[Annotated[str, Field(strict=True, max_length=70)]]] = None
    is_visible: Optional[StrictBool] = None
    code: Optional[StrictStr] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    __properties: ClassVar[List[str]] = ["id", "organization", "plugin_interface", "integration", "implemented_capabilities", "is_enabled", "is_official", "is_mine", "organization_slug", "capabilities", "is_visible", "code", "created_at", "updated_at"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Plugin from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "id",
            "organization",
            "implemented_capabilities",
            "is_enabled",
            "is_official",
            "is_mine",
            "organization_slug",
            "created_at",
            "updated_at",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of plugin_interface
        if self.plugin_interface:
            _dict['plugin_interface'] = self.plugin_interface.to_dict()
        # override the default output from pydantic by calling `to_dict()` of integration
        if self.integration:
            _dict['integration'] = self.integration.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Plugin from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "organization": obj.get("organization"),
            "plugin_interface": PluginPluginInterface.from_dict(obj["plugin_interface"]) if obj.get("plugin_interface") is not None else None,
            "integration": PluginIntegration.from_dict(obj["integration"]) if obj.get("integration") is not None else None,
            "implemented_capabilities": obj.get("implemented_capabilities"),
            "is_enabled": obj.get("is_enabled"),
            "is_official": obj.get("is_official"),
            "is_mine": obj.get("is_mine"),
            "organization_slug": obj.get("organization_slug"),
            "capabilities": obj.get("capabilities"),
            "is_visible": obj.get("is_visible"),
            "code": obj.get("code"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at")
        })
        return _obj





# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from automio.models.action_type_enum import ActionTypeEnum
from typing import Optional, Set
from typing_extensions import Self

class Action(BaseModel):
    """
    Action
    """ # noqa: E501
    id: Optional[StrictInt] = None
    is_mine: Optional[StrictBool] = None
    table_name: Optional[StrictStr] = None
    can_view_code: Optional[StrictBool] = None
    action_type: Optional[ActionTypeEnum] = None
    name: Annotated[str, Field(strict=True, max_length=255)]
    slug: Optional[Annotated[str, Field(strict=True)]] = None
    description: Optional[StrictStr] = None
    required_confirmation: Optional[StrictBool] = None
    code: Optional[StrictStr] = None
    input_schema: Optional[Any] = None
    is_public: Optional[StrictBool] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    organization: Optional[StrictInt] = None
    table: Optional[StrictInt] = None
    app: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = ["id", "is_mine", "table_name", "can_view_code", "action_type", "name", "slug", "description", "required_confirmation", "code", "input_schema", "is_public", "created_at", "updated_at", "organization", "table", "app"]

    @field_validator('slug')
    def slug_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[-a-zA-Z0-9_]+$", value):
            raise ValueError(r"must validate the regular expression /^[-a-zA-Z0-9_]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Action from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "id",
            "is_mine",
            "table_name",
            "can_view_code",
            "slug",
            "created_at",
            "updated_at",
            "organization",
            "table",
            "app",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if table_name (nullable) is None
        # and model_fields_set contains the field
        if self.table_name is None and "table_name" in self.model_fields_set:
            _dict['table_name'] = None

        # set to None if input_schema (nullable) is None
        # and model_fields_set contains the field
        if self.input_schema is None and "input_schema" in self.model_fields_set:
            _dict['input_schema'] = None

        # set to None if table (nullable) is None
        # and model_fields_set contains the field
        if self.table is None and "table" in self.model_fields_set:
            _dict['table'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Action from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "is_mine": obj.get("is_mine"),
            "table_name": obj.get("table_name"),
            "can_view_code": obj.get("can_view_code"),
            "action_type": obj.get("action_type"),
            "name": obj.get("name"),
            "slug": obj.get("slug"),
            "description": obj.get("description"),
            "required_confirmation": obj.get("required_confirmation"),
            "code": obj.get("code"),
            "input_schema": obj.get("input_schema"),
            "is_public": obj.get("is_public"),
            "created_at": obj.get("created_at"),
            "updated_at": obj.get("updated_at"),
            "organization": obj.get("organization"),
            "table": obj.get("table"),
            "app": obj.get("app")
        })
        return _obj





# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.api_key_distribution_enum import ApiKeyDistributionEnum

class TestApiKeyDistributionEnum(unittest.TestCase):
    """ApiKeyDistributionEnum unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testApiKeyDistributionEnum(self):
        """Test ApiKeyDistributionEnum"""
        # inst = ApiKeyDistributionEnum()

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    RobotnikAI API

    Documentation of API endpoints of RobotnikAI for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from robotnikai.models.paginated_integration_list import PaginatedIntegrationList

class TestPaginatedIntegrationList(unittest.TestCase):
    """PaginatedIntegrationList unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaginatedIntegrationList:
        """Test PaginatedIntegrationList
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaginatedIntegrationList`
        """
        model = PaginatedIntegrationList()
        if include_optional:
            return PaginatedIntegrationList(
                count = 123,
                next = 'http://api.example.org/accounts/?page=4',
                previous = 'http://api.example.org/accounts/?page=2',
                results = [
                    robotnikai.models.integration.Integration(
                        id = 56, 
                        integration_id = '', 
                        type = null, 
                        name = '', 
                        tags = [
                            ''
                            ], 
                        categories = [
                            robotnikai.models.integration_category.IntegrationCategory(
                                id = 56, 
                                name = '', 
                                slug = 'z', )
                            ], 
                        version = '', 
                        icon = '', 
                        description = '', 
                        website_url = '', 
                        account_fields = null, 
                        dependencies = null, 
                        configuration_form = null, 
                        activated = True, 
                        owner = 56, 
                        connected_accounts = null, 
                        api_base_url = '', 
                        account = null, 
                        openapi_json_scheme_url = '', 
                        api_docs_url = '', 
                        countries = [
                            ''
                            ], 
                        is_global = True, 
                        is_sandbox = True, 
                        api_schema = '', 
                        is_official = True, 
                        client_id = '', 
                        secret = '', 
                        access_token_url = '', 
                        authorize_url = '', 
                        identity_url = '', 
                        scope = '', 
                        oauth_basic_auth = True, 
                        oauth_pkce_enabled = True, 
                        accept_header = '', 
                        api_key = '', 
                        api_key_distribution = null, 
                        api_key_query_param_key = '', 
                        api_key_body_param_key = '', 
                        api_auth_header_name = '', 
                        api_auth_header_key = '', 
                        hmac_algorithm = 'sha1', 
                        hmac_fields = [
                            ''
                            ], 
                        hmac_separator = '', 
                        hmac_header_placeholder = '', 
                        hmac_custom_signature_field = '', 
                        hmac_key_format = 'base64', 
                        status_check_url = '', 
                        wsdl_document_url = '', 
                        wsdl_auth_method = '', 
                        wsdl_custom_auth = '', 
                        extra_headers = null, )
                    ]
            )
        else:
            return PaginatedIntegrationList(
                count = 123,
                results = [
                    robotnikai.models.integration.Integration(
                        id = 56, 
                        integration_id = '', 
                        type = null, 
                        name = '', 
                        tags = [
                            ''
                            ], 
                        categories = [
                            robotnikai.models.integration_category.IntegrationCategory(
                                id = 56, 
                                name = '', 
                                slug = 'z', )
                            ], 
                        version = '', 
                        icon = '', 
                        description = '', 
                        website_url = '', 
                        account_fields = null, 
                        dependencies = null, 
                        configuration_form = null, 
                        activated = True, 
                        owner = 56, 
                        connected_accounts = null, 
                        api_base_url = '', 
                        account = null, 
                        openapi_json_scheme_url = '', 
                        api_docs_url = '', 
                        countries = [
                            ''
                            ], 
                        is_global = True, 
                        is_sandbox = True, 
                        api_schema = '', 
                        is_official = True, 
                        client_id = '', 
                        secret = '', 
                        access_token_url = '', 
                        authorize_url = '', 
                        identity_url = '', 
                        scope = '', 
                        oauth_basic_auth = True, 
                        oauth_pkce_enabled = True, 
                        accept_header = '', 
                        api_key = '', 
                        api_key_distribution = null, 
                        api_key_query_param_key = '', 
                        api_key_body_param_key = '', 
                        api_auth_header_name = '', 
                        api_auth_header_key = '', 
                        hmac_algorithm = 'sha1', 
                        hmac_fields = [
                            ''
                            ], 
                        hmac_separator = '', 
                        hmac_header_placeholder = '', 
                        hmac_custom_signature_field = '', 
                        hmac_key_format = 'base64', 
                        status_check_url = '', 
                        wsdl_document_url = '', 
                        wsdl_auth_method = '', 
                        wsdl_custom_auth = '', 
                        extra_headers = null, )
                    ],
        )
        """

    def testPaginatedIntegrationList(self):
        """Test PaginatedIntegrationList"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from automio.models.invoke_parallel_call_for_connection_request_data_list_inner import InvokeParallelCallForConnectionRequestDataListInner
from typing import Optional, Set
from typing_extensions import Self

class InvokeParallelCallForConnectionRequest(BaseModel):
    """
    InvokeParallelCallForConnectionRequest
    """ # noqa: E501
    method: StrictStr = Field(description="HTTP method to use for each request")
    endpoint: Optional[StrictStr] = Field(default=None, description="API Endpoint to call for each data object")
    data_list: List[InvokeParallelCallForConnectionRequestDataListInner] = Field(description="List of data objects to process in parallel")
    request_headers: Dict[str, Any] = Field(description="Headers to include in the request to API Service")
    streaming: Optional[StrictBool] = Field(default=False, description="Enable streaming response to return results as they become available")
    __properties: ClassVar[List[str]] = ["method", "endpoint", "data_list", "request_headers", "streaming"]

    @field_validator('method')
    def method_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['GET', 'POST', 'PUT', 'PATCH', 'DELETE']):
            raise ValueError("must be one of enum values ('GET', 'POST', 'PUT', 'PATCH', 'DELETE')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of InvokeParallelCallForConnectionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data_list (list)
        _items = []
        if self.data_list:
            for _item_data_list in self.data_list:
                if _item_data_list:
                    _items.append(_item_data_list.to_dict())
            _dict['data_list'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InvokeParallelCallForConnectionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "method": obj.get("method"),
            "endpoint": obj.get("endpoint"),
            "data_list": [InvokeParallelCallForConnectionRequestDataListInner.from_dict(_item) for _item in obj["data_list"]] if obj.get("data_list") is not None else None,
            "request_headers": obj.get("request_headers"),
            "streaming": obj.get("streaming") if obj.get("streaming") is not None else False
        })
        return _obj





# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class StatusEnum(str, Enum):
    """
    * `pending` - pending * `running` - running * `completed` - completed * `failed` - failed * `stopped` - stopped
    """

    """
    allowed enum values
    """
    PENDING = 'pending'
    RUNNING = 'running'
    COMPLETED = 'completed'
    FAILED = 'failed'
    STOPPED = 'stopped'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of StatusEnum from a JSON string"""
        return cls(json.loads(json_str))





# flake8: noqa

# import apis into api package
from automio.api.actions_api import ActionsApi
from automio.api.applications_tables_graph_ql_api import ApplicationsTablesGraphQLApi
from automio.api.applications_tables_tags_api import ApplicationsTablesTagsApi
from automio.api.integrations_api import IntegrationsApi
from automio.api.integrations_api_calls_api import IntegrationsAPICallsApi
from automio.api.integrations_email_service_api import IntegrationsEmailServiceApi
from automio.api.plugin_interfaces_plugins_api import PluginInterfacesPluginsApi
from automio.api.plugins_api import PluginsApi




# coding: utf-8

"""
    Automio API

    Documentation of API endpoints of Automio for SDK

    The version of the OpenAPI document: 0.1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from automio.models.api_key_distribution_enum import ApiKeyDistributionEnum
from automio.models.hmac_algorithm_enum import HmacAlgorithmEnum
from automio.models.hmac_key_format_enum import HmacKeyFormatEnum
from automio.models.integration_category import IntegrationCategory
from automio.models.integration_type import IntegrationType
from typing import Optional, Set
from typing_extensions import Self

class Integration(BaseModel):
    """
    Integration
    """ # noqa: E501
    id: Optional[StrictInt] = None
    integration_id: Annotated[str, Field(strict=True, max_length=100)]
    type: Optional[IntegrationType] = None
    name: Annotated[str, Field(strict=True, max_length=150)]
    tags: Optional[List[StrictStr]] = None
    categories: Optional[List[IntegrationCategory]] = None
    version: Annotated[str, Field(strict=True, max_length=10)]
    icon: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    website_url: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    account_fields: Optional[Any] = None
    dependencies: Optional[Any] = None
    configuration_form: Optional[Any] = None
    activated: Optional[StrictBool] = None
    owner: Optional[StrictInt] = None
    connected_accounts: Optional[Any] = None
    api_base_url: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    account: Optional[Any] = None
    openapi_json_scheme_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    api_docs_url: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    countries: Optional[List[Annotated[str, Field(strict=True, max_length=2)]]] = None
    is_global: Optional[StrictBool] = Field(default=None, description="Is the integration available globally? If true, countries field will be ignored.")
    is_sandbox: Optional[StrictBool] = False
    api_schema: Optional[StrictStr] = None
    is_official: Optional[StrictBool] = None
    client_id: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    secret: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    access_token_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    authorize_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    identity_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    scope: Optional[StrictStr] = ''
    oauth_basic_auth: Optional[StrictBool] = False
    oauth_pkce_enabled: Optional[StrictBool] = False
    accept_header: Optional[Annotated[str, Field(strict=True, max_length=50)]] = None
    api_key: Optional[Annotated[str, Field(strict=True, max_length=200)]] = None
    api_key_distribution: Optional[ApiKeyDistributionEnum] = None
    api_key_query_param_key: Optional[Annotated[str, Field(strict=True, max_length=50)]] = None
    api_key_body_param_key: Optional[Annotated[str, Field(strict=True, max_length=50)]] = None
    api_auth_header_name: Optional[StrictStr] = None
    api_auth_header_key: Optional[Annotated[str, Field(strict=True, max_length=50)]] = None
    hmac_algorithm: Optional[HmacAlgorithmEnum] = None
    hmac_fields: Optional[List[Annotated[str, Field(strict=True, max_length=50)]]] = None
    hmac_separator: Optional[Annotated[str, Field(strict=True, max_length=20)]] = None
    hmac_header_placeholder: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    hmac_custom_signature_field: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    hmac_key_format: Optional[HmacKeyFormatEnum] = None
    status_check_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    wsdl_document_url: Optional[Annotated[str, Field(strict=True, max_length=250)]] = None
    wsdl_auth_method: Optional[Annotated[str, Field(strict=True, max_length=20)]] = None
    wsdl_custom_auth: Optional[Annotated[str, Field(strict=True, max_length=500)]] = None
    extra_headers: Optional[Any] = None
    __properties: ClassVar[List[str]] = ["id", "integration_id", "type", "name", "tags", "categories", "version", "icon", "description", "website_url", "account_fields", "dependencies", "configuration_form", "activated", "owner", "connected_accounts", "api_base_url", "account", "openapi_json_scheme_url", "api_docs_url", "countries", "is_global", "is_sandbox", "api_schema", "is_official", "client_id", "secret", "access_token_url", "authorize_url", "identity_url", "scope", "oauth_basic_auth", "oauth_pkce_enabled", "accept_header", "api_key", "api_key_distribution", "api_key_query_param_key", "api_key_body_param_key", "api_auth_header_name", "api_auth_header_key", "hmac_algorithm", "hmac_fields", "hmac_separator", "hmac_header_placeholder", "hmac_custom_signature_field", "hmac_key_format", "status_check_url", "wsdl_document_url", "wsdl_auth_method", "wsdl_custom_auth", "extra_headers"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Integration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "id",
            "type",
            "categories",
            "activated",
            "owner",
            "connected_accounts",
            "account",
            "is_official",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of type
        if self.type:
            _dict['type'] = self.type.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in categories (list)
        _items = []
        if self.categories:
            for _item_categories in self.categories:
                if _item_categories:
                    _items.append(_item_categories.to_dict())
            _dict['categories'] = _items
        # set to None if tags (nullable) is None
        # and model_fields_set contains the field
        if self.tags is None and "tags" in self.model_fields_set:
            _dict['tags'] = None

        # set to None if categories (nullable) is None
        # and model_fields_set contains the field
        if self.categories is None and "categories" in self.model_fields_set:
            _dict['categories'] = None

        # set to None if account_fields (nullable) is None
        # and model_fields_set contains the field
        if self.account_fields is None and "account_fields" in self.model_fields_set:
            _dict['account_fields'] = None

        # set to None if dependencies (nullable) is None
        # and model_fields_set contains the field
        if self.dependencies is None and "dependencies" in self.model_fields_set:
            _dict['dependencies'] = None

        # set to None if configuration_form (nullable) is None
        # and model_fields_set contains the field
        if self.configuration_form is None and "configuration_form" in self.model_fields_set:
            _dict['configuration_form'] = None

        # set to None if owner (nullable) is None
        # and model_fields_set contains the field
        if self.owner is None and "owner" in self.model_fields_set:
            _dict['owner'] = None

        # set to None if connected_accounts (nullable) is None
        # and model_fields_set contains the field
        if self.connected_accounts is None and "connected_accounts" in self.model_fields_set:
            _dict['connected_accounts'] = None

        # set to None if api_base_url (nullable) is None
        # and model_fields_set contains the field
        if self.api_base_url is None and "api_base_url" in self.model_fields_set:
            _dict['api_base_url'] = None

        # set to None if account (nullable) is None
        # and model_fields_set contains the field
        if self.account is None and "account" in self.model_fields_set:
            _dict['account'] = None

        # set to None if openapi_json_scheme_url (nullable) is None
        # and model_fields_set contains the field
        if self.openapi_json_scheme_url is None and "openapi_json_scheme_url" in self.model_fields_set:
            _dict['openapi_json_scheme_url'] = None

        # set to None if countries (nullable) is None
        # and model_fields_set contains the field
        if self.countries is None and "countries" in self.model_fields_set:
            _dict['countries'] = None

        # set to None if api_schema (nullable) is None
        # and model_fields_set contains the field
        if self.api_schema is None and "api_schema" in self.model_fields_set:
            _dict['api_schema'] = None

        # set to None if client_id (nullable) is None
        # and model_fields_set contains the field
        if self.client_id is None and "client_id" in self.model_fields_set:
            _dict['client_id'] = None

        # set to None if secret (nullable) is None
        # and model_fields_set contains the field
        if self.secret is None and "secret" in self.model_fields_set:
            _dict['secret'] = None

        # set to None if access_token_url (nullable) is None
        # and model_fields_set contains the field
        if self.access_token_url is None and "access_token_url" in self.model_fields_set:
            _dict['access_token_url'] = None

        # set to None if authorize_url (nullable) is None
        # and model_fields_set contains the field
        if self.authorize_url is None and "authorize_url" in self.model_fields_set:
            _dict['authorize_url'] = None

        # set to None if identity_url (nullable) is None
        # and model_fields_set contains the field
        if self.identity_url is None and "identity_url" in self.model_fields_set:
            _dict['identity_url'] = None

        # set to None if scope (nullable) is None
        # and model_fields_set contains the field
        if self.scope is None and "scope" in self.model_fields_set:
            _dict['scope'] = None

        # set to None if oauth_basic_auth (nullable) is None
        # and model_fields_set contains the field
        if self.oauth_basic_auth is None and "oauth_basic_auth" in self.model_fields_set:
            _dict['oauth_basic_auth'] = None

        # set to None if oauth_pkce_enabled (nullable) is None
        # and model_fields_set contains the field
        if self.oauth_pkce_enabled is None and "oauth_pkce_enabled" in self.model_fields_set:
            _dict['oauth_pkce_enabled'] = None

        # set to None if accept_header (nullable) is None
        # and model_fields_set contains the field
        if self.accept_header is None and "accept_header" in self.model_fields_set:
            _dict['accept_header'] = None

        # set to None if api_key (nullable) is None
        # and model_fields_set contains the field
        if self.api_key is None and "api_key" in self.model_fields_set:
            _dict['api_key'] = None

        # set to None if api_key_query_param_key (nullable) is None
        # and model_fields_set contains the field
        if self.api_key_query_param_key is None and "api_key_query_param_key" in self.model_fields_set:
            _dict['api_key_query_param_key'] = None

        # set to None if api_key_body_param_key (nullable) is None
        # and model_fields_set contains the field
        if self.api_key_body_param_key is None and "api_key_body_param_key" in self.model_fields_set:
            _dict['api_key_body_param_key'] = None

        # set to None if api_auth_header_name (nullable) is None
        # and model_fields_set contains the field
        if self.api_auth_header_name is None and "api_auth_header_name" in self.model_fields_set:
            _dict['api_auth_header_name'] = None

        # set to None if api_auth_header_key (nullable) is None
        # and model_fields_set contains the field
        if self.api_auth_header_key is None and "api_auth_header_key" in self.model_fields_set:
            _dict['api_auth_header_key'] = None

        # set to None if hmac_algorithm (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_algorithm is None and "hmac_algorithm" in self.model_fields_set:
            _dict['hmac_algorithm'] = None

        # set to None if hmac_fields (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_fields is None and "hmac_fields" in self.model_fields_set:
            _dict['hmac_fields'] = None

        # set to None if hmac_separator (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_separator is None and "hmac_separator" in self.model_fields_set:
            _dict['hmac_separator'] = None

        # set to None if hmac_header_placeholder (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_header_placeholder is None and "hmac_header_placeholder" in self.model_fields_set:
            _dict['hmac_header_placeholder'] = None

        # set to None if hmac_custom_signature_field (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_custom_signature_field is None and "hmac_custom_signature_field" in self.model_fields_set:
            _dict['hmac_custom_signature_field'] = None

        # set to None if hmac_key_format (nullable) is None
        # and model_fields_set contains the field
        if self.hmac_key_format is None and "hmac_key_format" in self.model_fields_set:
            _dict['hmac_key_format'] = None

        # set to None if status_check_url (nullable) is None
        # and model_fields_set contains the field
        if self.status_check_url is None and "status_check_url" in self.model_fields_set:
            _dict['status_check_url'] = None

        # set to None if wsdl_document_url (nullable) is None
        # and model_fields_set contains the field
        if self.wsdl_document_url is None and "wsdl_document_url" in self.model_fields_set:
            _dict['wsdl_document_url'] = None

        # set to None if wsdl_auth_method (nullable) is None
        # and model_fields_set contains the field
        if self.wsdl_auth_method is None and "wsdl_auth_method" in self.model_fields_set:
            _dict['wsdl_auth_method'] = None

        # set to None if wsdl_custom_auth (nullable) is None
        # and model_fields_set contains the field
        if self.wsdl_custom_auth is None and "wsdl_custom_auth" in self.model_fields_set:
            _dict['wsdl_custom_auth'] = None

        # set to None if extra_headers (nullable) is None
        # and model_fields_set contains the field
        if self.extra_headers is None and "extra_headers" in self.model_fields_set:
            _dict['extra_headers'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Integration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "integration_id": obj.get("integration_id"),
            "type": IntegrationType.from_dict(obj["type"]) if obj.get("type") is not None else None,
            "name": obj.get("name"),
            "tags": obj.get("tags"),
            "categories": [IntegrationCategory.from_dict(_item) for _item in obj["categories"]] if obj.get("categories") is not None else None,
            "version": obj.get("version"),
            "icon": obj.get("icon"),
            "description": obj.get("description"),
            "website_url": obj.get("website_url"),
            "account_fields": obj.get("account_fields"),
            "dependencies": obj.get("dependencies"),
            "configuration_form": obj.get("configuration_form"),
            "activated": obj.get("activated"),
            "owner": obj.get("owner"),
            "connected_accounts": obj.get("connected_accounts"),
            "api_base_url": obj.get("api_base_url"),
            "account": obj.get("account"),
            "openapi_json_scheme_url": obj.get("openapi_json_scheme_url"),
            "api_docs_url": obj.get("api_docs_url"),
            "countries": obj.get("countries"),
            "is_global": obj.get("is_global"),
            "is_sandbox": obj.get("is_sandbox") if obj.get("is_sandbox") is not None else False,
            "api_schema": obj.get("api_schema"),
            "is_official": obj.get("is_official"),
            "client_id": obj.get("client_id"),
            "secret": obj.get("secret"),
            "access_token_url": obj.get("access_token_url"),
            "authorize_url": obj.get("authorize_url"),
            "identity_url": obj.get("identity_url"),
            "scope": obj.get("scope") if obj.get("scope") is not None else '',
            "oauth_basic_auth": obj.get("oauth_basic_auth") if obj.get("oauth_basic_auth") is not None else False,
            "oauth_pkce_enabled": obj.get("oauth_pkce_enabled") if obj.get("oauth_pkce_enabled") is not None else False,
            "accept_header": obj.get("accept_header"),
            "api_key": obj.get("api_key"),
            "api_key_distribution": obj.get("api_key_distribution"),
            "api_key_query_param_key": obj.get("api_key_query_param_key"),
            "api_key_body_param_key": obj.get("api_key_body_param_key"),
            "api_auth_header_name": obj.get("api_auth_header_name"),
            "api_auth_header_key": obj.get("api_auth_header_key"),
            "hmac_algorithm": obj.get("hmac_algorithm"),
            "hmac_fields": obj.get("hmac_fields"),
            "hmac_separator": obj.get("hmac_separator"),
            "hmac_header_placeholder": obj.get("hmac_header_placeholder"),
            "hmac_custom_signature_field": obj.get("hmac_custom_signature_field"),
            "hmac_key_format": obj.get("hmac_key_format"),
            "status_check_url": obj.get("status_check_url"),
            "wsdl_document_url": obj.get("wsdl_document_url"),
            "wsdl_auth_method": obj.get("wsdl_auth_method"),
            "wsdl_custom_auth": obj.get("wsdl_custom_auth"),
            "extra_headers": obj.get("extra_headers")
        })
        return _obj


